@@ -1,9 +1,9 @@
 #!/usr/bin/perl
 # -------------------------------------------------------
-# $Header: /home/cvs/dobackup/dobackup.pl,v 4.22 2002/12/03 17:24:37 imorgan Exp $
+# $Header: /home/cvs/dobackup/dobackup.pl,v 4.28 2004/01/09 23:16:52 imorgan Exp $
 # -------------------------------------------------------
 #
-# Copyright 1997-2002 Webcon, Inc.
+# Copyright 1997-2004 Webcon, Inc.
 #
 # This code is available for public use under the terms of the GNU Public
 # License (GPL). See http://www.gnu.org/copyleft/gpl.html for license
@@ -12,12 +12,12 @@
 # We reserve the right to modify the licensing terms for any future version
 # of this product.
 #
-# Product website: http://www.webcon.net/opensource/dobackup/
+# Product website: http://www.webcon.ca/opensource/dobackup/
 #
-# Original Program by: Robert Hardy <rhardy@webcon.net>
+# Original Program by: Robert Hardy <rhardy@webcon.ca>
 # Prior to first public release, all updates by:
-#   Robert Hardy <rhardy@webcon.net>, and
-#   Ian Morgan <imorgan@webcon.net>
+#   Robert Hardy <rhardy@webcon.ca>, and
+#   Ian Morgan <imorgan@webcon.ca>
 #
 
 #Becuase we use the new "our" function for global declaration.
@@ -93,8 +93,8 @@ our($Debug); $Debug=0; # Debug off by default
 our($ExecutableName); ($ExecutableName)=( $0 =~ /([^\/]*)$/ );
 
 # ------------- Defaults overrideable by Config File
-our($BackupProg);      $BackupProg = "/usr/bin/afio -o -T 3k -G 9 -Z";
-our($BackupProgDebug); $BackupProgDebug = "/usr/bin/afio -o -v -T 3k -G 9 -z -Z";
+our($BackupProg);      $BackupProg = "/usr/bin/afio -o -T 10k -G 9 -Z -x -1 a";
+our($BackupProgDebug); $BackupProgDebug = "/usr/bin/afio -o -v -T 10k -G 9 -z -Z -x -1 a";
 our($Cat);             $Cat = "/bin/cat";
 our($Cut);             $Cut = "/usr/bin/cut";
 our($DF);              $DF = "/bin/df";
@@ -102,7 +102,7 @@ our($Echo);            $Echo = "/bin/echo";
 our($MT);              $MT = "/bin/mt";
 our($MV);              $MV = "/bin/mv";
 our($MailProg);        $MailProg = "/usr/sbin/sendmail";
-our($Ping);            $Ping = "/bin/ping -q -c 1";
+our($Ping);            $Ping = "/bin/ping -q -c 1 -w 10 -W 10";
 our($TR);              $TR = "/usr/bin/tr";
 our($Tail);            $Tail = "/usr/bin/tail";
 our($Mount);           $Mount = "/bin/mount";
@@ -205,7 +205,7 @@ sub ShowSyntax {
   my($temp)=('');
 
   print "\nUsage: $ExecutableName <options>\n";
-  $temp = '$Revision: 4.22 $';
+  $temp = '$Revision: 4.28 $';
   ($temp) = ( $temp =~ /^\$Revision\:\ (.*?)\ \$$/ );
   print 'Version: ',"$temp\n";
   print <<EOF;
@@ -538,6 +538,7 @@ sub PrepareBackup{
   if (-e "$BackupErrorFile") {unlink "$BackupErrorFile";}
 
   open (BACKTEMP, ">$ToBackupTempFile") || die "Couldn\'t Create ToBackupTempFile: $ToBackupTempFile\n";
+  binmode ( BACKTEMP, ":raw" );
   if ( (!$opt_fullbackup && $Timestamp{$Host} > 0) or ($TimeStampOverride && $TimeStampOverride > 0) ) { # Prepare Incremental Backup Build
     if ( $TimeStampOverride && $TimeStampOverride > 0 ) {
       $StampCTime=$TimeStampOverride;
@@ -556,6 +557,7 @@ sub PrepareBackup{
       if ( -d "$ToBackup" && (! -l "$ToBackup") ) {
         &ProcessDir("$ToBackup","$StampCTime");
       }
+      # Handle case where an individual file is listed in the list of paths to backup.
       elsif ( ((undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,$CTime) = stat("$ToBackup")) && ($CTime >= $StampCTime) ) {
         print BACKTEMP "$ToBackup\n";
       }
@@ -602,15 +604,14 @@ sub PrepareBackup{
 sub ProcessDir {
   my($Path)=$_[0];
   my($CTime)=$_[1];
-  my($TempCTime);
-  my(@SubDirs)=();
-  my(@NonDirs)=();
-  my($DirCount)=0;
+  my($TempCTime)=0;
+  my($DirElement)='';
   my($SubDebug)=0;
   my($FileOwner,$DirOwner)=(0,0);
-  my($pat)=('');
+  my($pat)='';
   study "$Path";
 
+  # Check for and process .nobackup files
   if ( -f "$Path/\.nobackup" ) {
     (undef,undef,undef,undef,$FileOwner,undef,undef,undef,undef,undef,undef,undef,undef) = stat("$Path/\.nobackup");
     (undef,undef,undef,undef,$DirOwner,undef,undef,undef,undef,undef,undef,undef,undef) = stat("$Path");
@@ -650,6 +651,7 @@ sub ProcessDir {
     }
   }
 
+  # Process the exclusion lists
   foreach (@{$Excludes{$Host}}) {
     if ( defined ($_) ) { #take care of "empty" array.
       if ( $Path =~ qr/$_/ ) {
@@ -661,74 +663,54 @@ sub ProcessDir {
     }
   }
 
+  # Read the contents of the directory
   opendir(TOPROCESS, "$Path");
-  foreach (grep !/^\.\.?$/, readdir TOPROCESS) {
-    if (-d "$Path/$_" && (! -l "$Path/$_") ) { push @SubDirs, "$Path/$_";}
-    else { push @NonDirs, "$Path/$_"; }
-    ++$DirCount;
-  }
-  if ( $DirCount == 0 ) {
-    (undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,$TempCTime) = stat("$Path");
-    if ( $TempCTime >= $CTime ) {
-      print BACKTEMP "$Path\n";
-      ($Debug > 1) && print "Adding $Path\n";
-    }
-    return;
-  }
+ELEMENT:
+  foreach (grep !/^\.\.?$/, readdir TOPROCESS) { # Exclude "." and ".."
 
-  if ( $CTime == 0 ) {
-LOOP1:    foreach (@NonDirs) {
-      foreach $pat (@{$Excludes{$Host}}) {
-        if ( defined ($pat) ) {
-          if ( $_ =~ qr/$pat/ ) {
-            $Debug && print "Excluding $_ (matches $pat)\n";
-            next LOOP1;
-          } else {
-            ($Debug > 2) && print "Skipping $_ (does not match $pat)\n";
-          }
+    $DirElement="$Path/$_";
+
+    foreach $pat (@{$Excludes{$Host}}) {
+      if ( defined ($pat) ) {
+        if ( $DirElement =~ qr/$pat/ ) {
+          $Debug && print "Excluding $DirElement (matches $pat)\n";
+          next ELEMENT;
+        } else {
+          ($Debug > 2) && print "Skipping $DirElement (does not match $pat)\n";
         }
       }
-      print BACKTEMP "$_\n";
-      ($Debug > 1) && print "Adding $_\n";
     }
-  } else { # Must check CTimes
-LOOP2:    foreach (@NonDirs) {
-      (undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,$TempCTime) = stat("$_");
 
-      foreach $pat (@{$Excludes{$Host}}) {
-        if ( defined ($pat) ) {
-          if ( $_ =~ qr/$pat/ ) {
-            $Debug && print "Excluding $_ (matches $pat)\n";
-            next LOOP2;
-          } else {
-            ($Debug > 2) && print "Skipping $_ (does not match $pat)\n";
-          }
-        }
-      }
+    if ( $CTime != 0 ) { # Incremental backup
+      (undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,$TempCTime) = stat("$DirElement");
 
       # If it is a broken link it will have an empty CTime, so stat the link itself.
-      if ( (! $TempCTime) && (-l "$_") ) {
-        (undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,$TempCTime) = lstat("$_");
-        ($Debug > 2) && print "Checking broken link: $_\n";
+      if ( (! $TempCTime) && (-l "$DirElement") ) {
+        (undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,$TempCTime) = lstat("$DirElement");
+        ($Debug > 2) && print "Checking broken link: $DirElement\n";
       }
 
       # Would this _ever_ happen?!
-      if ( ! $TempCTime ) { # Always backup files non-links with Empty CTimes
-        print "TempCTTime Empty for $_\n";
-        print BACKTEMP "$_\n";
-        ($Debug > 1) && print "Adding $_\n";
+      if ( ! $TempCTime ) { # Always backup files (non-links) with Empty CTimes
+        print "TempCTTime Empty for $DirElement\n";
+        print BACKTEMP "$DirElement\n";
+        ($Debug > 1) && print "Adding $DirElement\n";
         next;
       }
 
       if ( $TempCTime >= $CTime ) {
-        print BACKTEMP "$_\n";
-        ($Debug > 1) && print "Adding $_\n";
+        print BACKTEMP "$DirElement\n";
+        ($Debug > 1) && print "Adding $DirElement\n";
       }
+    } else { # Full backup
+      print BACKTEMP "$DirElement\n";
+      ($Debug > 1) && print "Adding $DirElement\n";
     }
-  }
-  undef @NonDirs;
-  foreach (@SubDirs) {
-    &ProcessDir($_,$CTime);
+
+    if (-d "$DirElement" && (! -l "$DirElement") ) {
+      &ProcessDir("$DirElement",$CTime);
+    }
+  
   }
   return;
 }
@@ -891,12 +873,16 @@ sub PerformBackup{
 
   if( $Return == 0 ) {} # no problem
   elsif ($Return == 0xff00 ) {
-     print("ERROR: cat ($Cat) to backup command (",$Debug?$BackupProgDebug:$BackupProg,") failed!\n\nAborting backup!\n");
-     exit 1;
+    print("ERROR: cat ($Cat) to backup command (",$Debug?$BackupProgDebug:$BackupProg,") failed!\n\nAborting backup!\n");
+    print("Error log follows:\n");
+    print(`$Cat $BackupErrorFile.$Host`);
+    exit 1;
   } elsif ($Return > 0x80) {
-     $Return >>= 8;
-     print("ERROR: cat ($Cat) to backup command (",$Debug?$BackupProgDebug:$BackupProg,") ran with non-zero exit status $Return\n\nAborting backup!\n");
-     exit 1;
+    $Return >>= 8;
+    print("ERROR: cat ($Cat) to backup command (",$Debug?$BackupProgDebug:$BackupProg,") ran with non-zero exit status $Return\n\nAborting backup!\n");
+    print("Error log follows:\n");
+    print(`$Cat $BackupErrorFile.$Host`);
+    exit 1;
   } else {
     print "ERROR: cat ($Cat) to backup command (",$Debug?$BackupProgDebug:$BackupProg,") ran with";
     if ($Return & 0x80){
@@ -904,6 +890,8 @@ sub PerformBackup{
       print "coredump from ";
     }
     print "signal $Return\n\nAborting backup!\n";
+    print("Error log follows:\n");
+    print(`$Cat $BackupErrorFile.$Host`);
     exit 1;
   }
 
@@ -1018,7 +1006,7 @@ sub Cleanup{
   open (MAILTHIS, "|$MailProg $BackupAdministrator");
   print MAILTHIS "To: $BackupAdministrator\nSubject: $Host ",($Debug?"Debug-mode":"Automated")," Backup Summary\n\n";
 
-  $temp = '$Revision: 4.22 $';
+  $temp = '$Revision: 4.28 $';
   ($temp) = ( $temp =~ /^\$Revision\:\ (.*?)\ \$$/ );
 
   print MAILTHIS "This is an automated backup report from $ExecutableName v$temp\n\n";
@@ -1285,9 +1273,11 @@ sub RotateMedia {
       if ($Return != 0) {
         print "Error: failure unmounting $temp during rotation\n";
       } else {
-        $Return = 0xffff & system("$Eject $temp");
-        if ($Return != 0) {
-          print "Error: failure ejecting $temp during rotation\n";
+        if ($Eject ne '') {
+          $Return = 0xffff & system("$Eject $temp");
+          if ($Return != 0) {
+            print "Error: failure ejecting $temp during rotation\n";
+          }
         }
       }
     }
@@ -1297,7 +1287,7 @@ sub RotateMedia {
 
   if ($actions ne '') {
     open (MAILTHIS, "|$MailProg $BackupAdministrator");
-    print MAILTHIS "To: $BackupAdministrator\nSubject: Automated Backup Media Rotated\n\n";
+    print MAILTHIS "To: $BackupAdministrator\nSubject: Automated Backup Media Rotatation\n\n";
     print MAILTHIS "Note: This message is automatically generated by $ExecutableName\n\n";
     print MAILTHIS "The following actions need to be taken:\n---\n";
     print MAILTHIS $actions;
